@@ -62,7 +62,7 @@ static const gnutls_error_entry error_entries[] = {
 	ERROR_ENTRY(N_("An algorithm that is not enabled was negotiated."),
 		    GNUTLS_E_UNWANTED_ALGORITHM),
 	ERROR_ENTRY(N_
-		    ("A record packet with illegal version was received."),
+		    ("A packet with illegal or unsupported version was received."),
 		    GNUTLS_E_UNSUPPORTED_VERSION_PACKET),
 	ERROR_ENTRY(N_
 		    ("The Diffie-Hellman prime sent by the server is not acceptable (not long enough)."),
@@ -83,6 +83,8 @@ static const gnutls_error_entry error_entries[] = {
 		    GNUTLS_E_FATAL_ALERT_RECEIVED),
 	ERROR_ENTRY(N_("An unexpected TLS packet was received."),
 		    GNUTLS_E_UNEXPECTED_PACKET),
+	ERROR_ENTRY(N_("Failed to import the key into store."),
+		    GNUTLS_E_KEY_IMPORT_FAILED),
 	ERROR_ENTRY(N_
 		    ("An error was encountered at the TLS Finished packet calculation."),
 		    GNUTLS_E_ERROR_IN_FINISHED_PACKET),
@@ -385,6 +387,8 @@ static const gnutls_error_entry non_fatal_error_entries[] = {
 	ERROR_ENTRY(N_("Function was interrupted."), GNUTLS_E_INTERRUPTED),
 	ERROR_ENTRY(N_("Rehandshake was requested by the peer."),
 		    GNUTLS_E_REHANDSHAKE),
+	ERROR_ENTRY(N_("One of the involved algorithms has insufficient security level."),
+		    GNUTLS_E_INSUFFICIENT_SECURITY),
 	{NULL, NULL, 0}
 };
 
